import firestore, { Filter } from '@react-native-firebase/firestore';
export class StoreModel {
    _table;
    _firestoreDB;
    constructor(table, firestoreDB) {
        this._firestoreDB = firestoreDB;
        this._table = this._firestoreDB.collection(table);
    }
    async find(id) {
        try {
            const docSnap = await this._table.doc(id).get();
            if (docSnap.exists) {
                return {
                    data: { ...docSnap.data(), reference: id },
                    message: "Document found",
                    status: 'success'
                };
            }
            return { message: 'Document does not exist', status: 'success' };
        }
        catch (_) {
            return { message: 'Document does not exist', status: 'error' };
        }
    }
    async findAll(ids) {
        try {
            if (ids) {
                const results = [];
                for (let id of ids) {
                    const item = await this.find(id);
                    if (item.data) {
                        results.push(item);
                    }
                }
                return { data: results, message: 'Documents found', status: 'success' };
            }
            else {
                const snapshots = await this._table.get();
                if (!snapshots.empty) {
                    const data = snapshots.docs.map((document) => {
                        return { ...document.data(), reference: document.id };
                    });
                    return { data, message: 'Documents found', status: 'success' };
                }
                else {
                    return { message: 'Documents empty', status: 'success' };
                }
            }
        }
        catch (_) {
            return { message: 'Document does not exist', status: 'error' };
        }
    }
    async findWhereOrAnd({ wh, lim, order, offset }) {
        try {
            const andWhere = [];
            const orWhere = [];
            let queryRef = this._table;
            if (wh) {
                wh.parameter.forEach((clause) => {
                    const { key, operator, value } = clause;
                    const whe = Filter(key, operator, value);
                    clause.type === 'and' ?
                        andWhere.push(whe) : orWhere.push(whe);
                });
                if (wh.type === 'andOr') {
                    queryRef = this._table.where(Filter.and(...andWhere, Filter.or(...orWhere)));
                }
                else if (wh.type === 'or') {
                    queryRef = Filter.or(...orWhere);
                }
                else {
                    queryRef = Filter.and(...andWhere);
                }
            }
            if (order) {
                queryRef.orderBy(order);
            }
            if (offset) {
                const off = await this._table.doc(offset).get();
                queryRef.startAfter(off.data());
            }
            if (lim) {
                queryRef.limit(lim);
            }
            const snapshot = await this._table.get();
            if (!snapshot.empty) {
                const data = snapshot.docs.map(document => {
                    return { ...document.data(), reference: document.id };
                });
                return { data, message: 'Documents found', status: 'success' };
            }
            else {
                return { message: 'No Documents found', status: 'success' };
            }
        }
        catch (_) {
            return { message: 'No Documents found', status: 'error' };
        }
    }
    async findWhere({ wh, lim, order, offset }) {
        try {
            const andWhere = [];
            wh ? wh.forEach((clause) => {
                const { key, operator, value } = clause;
                const whe = Filter(key, operator, value);
                andWhere.push(whe);
            }) : [];
            if (wh) {
                this._table.where(Filter.and(...andWhere));
            }
            if (order) {
                this._table.orderBy(order);
            }
            if (offset) {
                const off = await this._table.doc(offset).get();
                this._table.startAfter(off.data());
            }
            if (lim) {
                this._table.limit(lim);
            }
            const snapshot = await this._table.get();
            if (!snapshot.empty) {
                const data = snapshot.docs.map(document => {
                    return { ...document.data(), reference: document.id };
                });
                return { data, message: 'Documents found', status: 'success' };
            }
            else {
                return { message: 'No Documents found', status: 'success' };
            }
        }
        catch (error) {
            return { message: 'No Documents found', status: 'error' };
        }
    }
    async save(data, id) {
        delete data.reference;
        try {
            if (id === undefined) {
                const documentRef = await this._table.add(data);
                return { status: 'success', message: 'Document added successfully!', data: documentRef.id };
            }
            else {
                await this._table.doc(id).set(data);
                return { status: 'success', message: 'Document updated successfully!', data: id };
            }
        }
        catch (_) {
            return { status: 'error', message: 'Unable to save document' };
        }
    }
    async delete(id) {
        try {
            await this._table.doc(id).delete();
            return { status: 'success', message: 'Document deleted successfully!' };
        }
        catch (error) {
            return { status: 'success', message: 'Unable to delete document' };
        }
    }
    async update(data, id) {
        try {
            delete data.reference;
            const result = await this._table.doc(id).update(data);
            return { data: result, message: 'Data updated successfully', status: 'success' };
        }
        catch (error) {
            return { message: 'Unable to update document', status: 'error' };
        }
    }
    stream(callBack, id) {
        try {
            if (id) {
                this._table.doc(id).onSnapshot((doc) => {
                    callBack({ ...doc.data(), reference: id });
                });
            }
            else {
                this._table.onSnapshot((coll) => {
                    callBack(coll.docs.map((value) => {
                        const data = { ...value.data(), reference: value.id };
                        return data;
                    }));
                });
            }
        }
        catch (_) {
            return;
        }
    }
    async streamWhere(wh, callBack, lim, order, offset) {
        try {
            const andWhere = [];
            wh ? wh.forEach((clause) => {
                const { key, operator, value } = clause;
                const whe = Filter(key, operator, value);
                andWhere.push(whe);
            }) : [];
            if (wh) {
                this._table.where(Filter.and(...andWhere));
            }
            if (order) {
                this._table.orderBy(order);
            }
            if (offset) {
                const off = await this._table.doc(offset).get();
                this._table.startAfter(off.data());
            }
            if (lim) {
                this._table.limit(lim);
            }
            this._table.onSnapshot((snapShot) => {
                callBack(snapShot.docs.map((value) => {
                    const data = { ...value.data(), reference: value.id };
                    return data;
                }));
            });
        }
        catch (error) {
            return;
        }
    }
    async countData(where) {
        try {
            const andWhere = [];
            where ? where.forEach((clause) => {
                const { key, operator, value } = clause;
                const whe = Filter(key, operator, value);
                andWhere.push(whe);
            }) : [];
            this._table.where(Filter.and(...andWhere));
            const data = await this._table.countFromServer().get();
            return { message: "Counted data", status: 'success', data: data.data() };
        }
        catch (_) {
            return { message: "Unable to cound data", status: "error" };
        }
    }
    saveBatch({ data }) {
        throw new Error("Method not implemented.");
    }
    updateBatch({ data }) {
        throw new Error("Method not implemented.");
    }
    deleteBatch({ ids }) {
        throw new Error("Method not implemented.");
    }
    async incrementDecrement({ dbReference, key, isIncrement, incrementalValue }) {
        try {
            const docRef = this._table.doc(dbReference);
            const value = isIncrement ? incrementalValue ?? 1 : (incrementalValue ?? 1) * -1;
            await docRef.update({ [key]: firestore.FieldValue.increment(value) });
            return { status: 'success', message: "Counter successful" };
        }
        catch (error) {
            return { status: 'error', message: "unable to count data" };
        }
    }
}
//# sourceMappingURL=StoreModel.js.map